<?php
namespace Drupal\admin_toolbar_content\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityAccessControlHandlerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ContentMenuLinkDerivative extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static();
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $links = [];

    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager */
    $entityTypeManager = \Drupal::service('entity_type.manager');

    $content_type_collections = \Drupal::service('module_handler')->invokeAll('content_type_collections');

    /** @var \Drupal\node\NodeTypeInterface[] $contentTypes */
    $contentTypes = $entityTypeManager->getStorage('node_type')->loadMultiple();

    foreach ($content_type_collections as $collection => $content_type_collection) {
      $links[$collection] = [
          'title' => t((string)$content_type_collection['label']),
          'route_name' => 'system.admin_content',
          'route_parameters' => [
            'collection' => $collection,
          ],
          'menu_name' => 'admin',
          'parent' => 'system.admin_content',
        ] + $base_plugin_definition;

      foreach ($content_type_collection['content_types'] as $content_type_id) {
        $this->addContentTypeLink($contentTypes[$content_type_id], $collection, $links, $base_plugin_definition);
        unset($contentTypes[$content_type_id]);
      }
    }

    $collection = 'content';
    foreach($contentTypes as $content_type) {
      $this->addContentTypeLink($content_type, $collection, $links, $base_plugin_definition);
    }

    return $links;
  }

  protected function addContentTypeLink($content_type, $collection, &$links, $base_plugin_definition) {

    $link_name = $collection . '.' . $content_type->id();

    $links[$link_name] = [
        'title' => $this->t($content_type->label()),
        'route_name' => 'system.admin_content',
        'route_parameters' => [
          'type' => $content_type->id(),
          'collection' => $collection,
        ],
        'menu_name' => 'admin',
        'parent' => !empty($links[$collection]) ? $base_plugin_definition['id'] . ':' . $collection :  "system.admin_content",
      ] + $base_plugin_definition;

    $links[$link_name . '.add'] = [
        'title' => $this->t('Add new'),
        'route_name' => "node.add",
        'route_parameters' => [
          'node_type' => $content_type->id()
        ],
        'menu_name' => 'admin',
        'parent' =>  $base_plugin_definition['id'] . ':' . $link_name
      ] + $base_plugin_definition;

  }
}